﻿using System;
using System.Windows.Forms;
using System.IO;
using System.Collections.Generic;
using System.Linq;

//***************************************
//*                                     *
//*  ANAGRAMMI BY CARLO BARABUCCI 2020  *
//*                                     *
//***************************************

namespace Anagrammi
{
    public partial class Form1 : Form
    {
        private List<string> Vocabolario;
        private string msgInterrogativi  = "i '?' verranno sostituiti con le lettere mancanti";
        private string msgAsterischi  = "*FiniscePer, IniziaPer*, *contiene*";
        private string msgAnagrammi  = "Parola senza accenti, poi invio, usa anche '?' e '*'";

        public Form1()
        {
            InitializeComponent();
            this.TextParola.KeyUp += new System.Windows.Forms.KeyEventHandler(this.TextParola_KeyUp);
            this.ListBox2.SelectedIndexChanged += new System.EventHandler(this.ListBox2_SelectedIndexChanged);
            this.ComboBox1.SelectedIndexChanged += new System.EventHandler(ComboBox1_SelectedIndexChanged);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // caricamento del vocabolario completo in una lista di stringhe
            Vocabolario = File.ReadAllLines(Application.StartupPath + "\\VocabolarioItaliano.txt").ToList();
            this.Text = "Anagrammi  ©CB2020    [" + Vocabolario.Count + " parole in linea]";
        }

        private void TextParola_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.KeyValue == 13)
            {
                TextParola.Text = TextParola.Text.ToLower().Trim(' ');
                RiempiCombo((byte)TextParola.Text.Length);
                if (TextParola.Text.Contains("?"))
                {
                    LabInfo.Text = msgInterrogativi;
                    SostituisciInterrogativi(TextParola.Text);
                }
                else if (TextParola.Text[TextParola.Text.Length - 1].ToString() == "*" && TextParola.Text[0].ToString() == "*")
                {
                    Contiene(TextParola.Text);
                }
                else if (TextParola.Text[0].ToString() == "*")
                {
                    FiniscePer(TextParola.Text);
                    LabInfo.Text = msgAsterischi;
                }
                else if (TextParola.Text[TextParola.Text.Length - 1].ToString() ==  "*")
                {
                    IniziaPer(TextParola.Text);
                    LabInfo.Text = msgAsterischi;
                }
                else
                {
                    LabInfo.Text = msgAnagrammi;
                    Ricerca(TextParola.Text);
                }
            }

            if (e.KeyValue == 219 && ModifierKeys == Keys.Shift)
            {
                LabInfo.Text = msgInterrogativi;
                if (TextParola.Text.Contains("*"))
                {
                    int pos = TextParola.SelectionStart;
                    TextParola.Text = TextParola.Text.Replace("*", "");
                    TextParola.SelectionStart = pos;
                }
            }

            if ((e.KeyValue == 187 && ModifierKeys == Keys.Shift) || e.KeyValue == 106)
            {
                LabInfo.Text = msgAsterischi;
                if (TextParola.Text.Contains("?"))
                {
                    int pos = TextParola.SelectionStart;
                    TextParola.Text = TextParola.Text.Replace("?", "");
                    TextParola.SelectionStart = pos;
                }
            }
        }

        private void TextParola_TextChanged(object sender, EventArgs e)
        {
            ButElimina.Enabled = false;
            ButAggiungi.Enabled = false;
            if (!TextParola.Text.Contains("?") && !TextParola.Text.Contains("*"))
            {
                LabInfo.Text = msgAnagrammi;
            }
        }

        private void ListBox2_SelectedIndexChanged(object sender, EventArgs e)
        {
            TextParola.Text = ListBox2.SelectedItem.ToString().ToLower();
            RiempiCombo((byte)TextParola.Text.Length);
            Ricerca(TextParola.Text);
        }

        private void ButAggiungi_Click(object sender, EventArgs e)
        {
            ButAggiungi.Enabled = false;
            if (TextParola.Text.Length > 0)
            {
                this.Enabled = false;
                RichTextBox1.ResetText();
                RichTextBox1.AppendText(TextParola.Text.ToLower() + Environment.NewLine);
                RichTextBox1.AppendText("aggiungo al vocabolario" + Environment.NewLine);
                RichTextBox1.Refresh();
                Vocabolario.Add(TextParola.Text.ToLower());
                RichTextBox1.AppendText("riordino vocabolario" + Environment.NewLine);
                RichTextBox1.Refresh();
                Vocabolario.Sort();
                RichTextBox1.AppendText("salvo vocabolario");
                RichTextBox1.Refresh();
                File.WriteAllLines(Application.StartupPath + "\\VocabolarioItaliano.txt", Vocabolario);
                this.Enabled = true;
            }
        }

        private void ButElimina_Click(object sender, EventArgs e)
        {
            ButElimina.Enabled = false;
            if (TextParola.Text.Length > 0)
            {
                Vocabolario.Remove(TextParola.Text.ToLower());
                RichTextBox1.ResetText();
                RichTextBox1.AppendText(TextParola.Text.ToLower() + Environment.NewLine);
                RichTextBox1.AppendText("rimosso/a dal vocabolario" + Environment.NewLine);
                RichTextBox1.Refresh();
                File.WriteAllLines(Application.StartupPath + "\\VocabolarioItaliano.txt", Vocabolario);
                RichTextBox1.AppendText("vocabolario salvato" + Environment.NewLine);
            }
        }
    
        private void ComboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            Ricerca(TextParola.Text);
            LabInfo.Text = msgAnagrammi;
        }

        private void RiempiCombo(byte NumCaratteri)
        {
            ComboBox1.Items.Clear();
            ComboBox1.Text = NumCaratteri.ToString();
            for (var Quante = NumCaratteri; Quante >= 2; Quante--)
            {
                ComboBox1.Items.Add(Quante);
            }
        }

        private void Ricerca(string ParolaCercata)
        {
            List<string> DatiLista = new List<string>();
            byte NumCaratteriOr = Convert.ToByte(ParolaCercata.Length);
            byte NumCaratteriCerca = Convert.ToByte(ComboBox1.Text);

            // logica....
            // ParolaCercata viene usata lettera per lettera per rimuovere da ParolaInAnalisi
            // le lettere una ad una, alla fine del ciclo se tutte le lettere di ParolaCercata,
            // erano presenti in ParolaInAnalisi, ParolaInAnalisi sarà vuota e era un anagramma.

            this.Enabled = false;
            foreach (var Elemento in Vocabolario)
            {
                // controllo solo le parole che hanno la stessa lunghezza della parola digitata, o dei caratteri scelti
                if (Elemento.Length == NumCaratteriCerca)
                {
                    string ParolaInAnalisi = Elemento; // duplico la parola del vocabolario in una variabile temporanea, perché la modificherò
                    for (var Cll = 0; Cll < NumCaratteriOr; Cll++) // ciclo lettera per lettera
                    {
                        Int16 pos = (Int16)ParolaInAnalisi.IndexOf(ParolaCercata[Cll], 0, ParolaInAnalisi.Length); // posizione della lettera attuale (ParolaCercata(PT)), nella parola in analisi (ParolaInAnalisi)
                        if (pos > -1)
                        {
                            ParolaInAnalisi = ParolaInAnalisi.Remove(pos, 1); // la lettera in analisi è contenuta nella parola del vocabolario (ParolaInAnalisi), la elimino
                        }
                    }
                    if (string.IsNullOrEmpty(ParolaInAnalisi)) // se non resta nessuna lettera, significa che la parala del vocabolario è un anagramma della parola digitata
                    {
                        if (NumCaratteriCerca < NumCaratteriOr) // se i caratteri cercati sono meno della lunghezza della parola digitata ci sarà una rimanenza
                        {
                            ParolaInAnalisi = ParolaCercata; // duplico la parola cercata in una variabile temporanea, perché la modificherò
                            for (var Cll = 0; Cll < NumCaratteriCerca; Cll++) // ciclo lettera per lettera
                            {
                                Int16 pos = (Int16)ParolaInAnalisi.IndexOf(Elemento[Cll], 0, ParolaInAnalisi.Length); // posizione della lettera attuale (ParolaCercata(PT)), nella parola in analisi (ParolaInAnalisi)
                                if (pos > -1)
                                {
                                    ParolaInAnalisi = ParolaInAnalisi.Remove(pos, 1); // la lettera in analisi è contenuta nella parola del vocabolario (ParolaInAnalisi), la elimino
                                }
                            }
                            // in ParolaInAnalisi restano le lettere in avanzo
                            DatiLista.Add(Elemento + "   [-" + ParolaInAnalisi + "]"); // mostro la parola del vocabolario che risponde al requisito di anagramma, con le lettere in eccesso
                        }
                        else
                        {
                            DatiLista.Add(Elemento); // mostro la parola del vocabolario che risponde al requisito di anagramma
                        }
                    }
                }
            }
            DatiLista.Add("- " + DatiLista.Count + " corrispondenze -");
            RichTextBox1.Lines = DatiLista.ToArray();
            if (NumCaratteriCerca == NumCaratteriOr) // solo se gli anagrammi sono della stessa lunghezza della parola digitata
            {
                // se la parola digitata (ParolaCercata) compare anche in ListBox1, significa che è contenuta nel vocabolario
                if (RichTextBox1.Text.Contains(ParolaCercata))
                {
                    // è contenuta la posso eliminare
                    ButAggiungi.Enabled = false;
                    ButElimina.Enabled = true;
                }
                else
                {
                    // non è contenuta la posso aggiungere
                    ButAggiungi.Enabled = true;
                    ButElimina.Enabled = false;
                }
            }
            FineElaborazione();
        }

        private void SostituisciInterrogativi(string ParolaCercata)
        {
            List<string> DatiLista = new List<string>();
            byte NumCaratteriOr = Convert.ToByte(ParolaCercata.Length);
            bool PrimoInterrogativo = true;
            string[,] divisa = new string[NumCaratteriOr + 1, 2];
            byte indice = 0; // indice delle stringhe, e relativa posizione
            byte LettereNote = 0;

            // logica....
            // esempio se ParolaCercata contiene: ma??i??iede
            // divisa[x,x] diventa:
            // divisa[0,0] = "ma" ' stringa che deve esistere
            // divisa[0,1] = "0" ' in posizione 0
            // divisa[1,0] = "i" ' stringa che deve esistere 
            // divisa[1,1] = "4" ' in posizione 4
            // divisa[2,0] = "iede" ' stringa che deve esistere 
            // divisa[2,1] = "7" ' in posizione 7

            divisa[0, 0] = ""; // inizializzo la posizione che serve se viene digitato un ? in prima posizione
            for (LettereNote = 0; LettereNote < NumCaratteriOr; LettereNote++) // ciclo delle lettere
            {
                // in divisa[indice,0] ci saranno le stringhe note, detratti i ?
                // in divisa[indice,1] ci saranno le posizioni di partenza delle stringhe note: LettereNote - divisa(indice, 0).Length
                if (ParolaCercata[LettereNote] == '?')
                {
                    if (PrimoInterrogativo)
                    {
                        divisa[indice, 1] = (LettereNote - divisa[indice, 0].Length).ToString();
                        indice += 1;
                    }
                    PrimoInterrogativo = false;
                }
                else
                {
                    divisa[indice, 0] += ParolaCercata[LettereNote];
                    PrimoInterrogativo = true;
                }
            }
            // se è stato digitato un ? alla fine della parola digitata, aggiusto indice per considerarlo nella ricerca
            if (ParolaCercata.Substring(ParolaCercata.Length - 1, 1) == "?")
            {
                indice -= 1;
            }
            else
            {
                divisa[indice, 1] = (LettereNote - divisa[indice, 0].Length).ToString();
            }
            // quando la matrice divisa è stata riempita con le stringhe note e relative posizioni
            // posso verificare che le parole in esame prese dal vocabolario le contengano tutte e nella giusta posizione
            this.Enabled = false;
            foreach (var Elemento in Vocabolario)
            {
                // controllo solo le parole che hanno la stessa lunghezza della parola digitata, compresi i ?
                if (Elemento.Length == NumCaratteriOr)
                {
                    bool Parola = true;
                    for (var test = 0; test <= indice; test++)
                    {
                        if (Elemento.Substring(Convert.ToInt16(divisa[test, 1]), (int)divisa[test, 0].Length) != divisa[test, 0]) // è sufficiente una non corrispondenza per uscire
                        {
                            Parola = false;
                            break;
                        }
                    }
                    if (Parola) // tutte le stringe sono presenti
                    {
                        DatiLista.Add(Elemento);
                    }
                }
            }
            DatiLista.Add("- " + DatiLista.Count + " corrispondenze -");
            RichTextBox1.Lines = DatiLista.ToArray();
            FineElaborazione();
        }

        private void Contiene(string ParolaCercata)
        {
            // ci sono due asterischi, uno all'inizio e uno alla fine
            List<string> DatiLista = new List<string>();
            ParolaCercata = ParolaCercata.Replace("*", "");
            this.Enabled = false;
            foreach (var Elemento in Vocabolario)
            {
                // mostro le parole che contengono una porzione della parola digitata
                if (Elemento.Contains(ParolaCercata))
                {
                    DatiLista.Add(Elemento);
                }
            }
            DatiLista.Add("- " + DatiLista.Count + " corrispondenze -");
            RichTextBox1.Lines = DatiLista.ToArray();
            FineElaborazione();
        }

        private void FiniscePer(string ParolaCercata)
        {
            // c'è un asterisco all'inizio
            List<string> DatiLista = new List<string>();
            ParolaCercata = ParolaCercata.Replace("*", "");
            this.Enabled = false;
            foreach (var Elemento in Vocabolario)
            {
                // escludo le parole più corte della parola digitata
                if (Elemento.Length >= ParolaCercata.Length)
                {
                    // mostro le parole con la giusta desinenza
                    if (Elemento.IndexOf(ParolaCercata, 0) == Elemento.Length - ParolaCercata.Length)
                    {
                        DatiLista.Add(Elemento);
                    }
                }
            }
            DatiLista.Add("- " + DatiLista.Count + " corrispondenze -");
            RichTextBox1.Lines = DatiLista.ToArray();
            FineElaborazione();
        }

        private void IniziaPer(string ParolaCercata)
        {
            // c'è un asterisco alla fine
            List<string> DatiLista = new List<string>();
            ParolaCercata = ParolaCercata.Replace("*", "");
            this.Enabled = false;
            foreach (var Elemento in Vocabolario)
            {
                // mostro le parole con il giusto inizio
                if (Elemento.IndexOf(ParolaCercata, 0) == 0)
                {
                    DatiLista.Add(Elemento);
                }
            }
            DatiLista.Add("- " + DatiLista.Count + " corrispondenze -");
            RichTextBox1.Lines = DatiLista.ToArray();
            FineElaborazione();
        }

        private void FineElaborazione()
        {
            this.Enabled = true;
            RichTextBox1.Select(0, 0);
            TextParola.Focus();
            TextParola.DeselectAll();
        }
    }
}
